#!/usr/bin/perl

# loadsubscribers.pl - This script loads a CSV-file
# (generated by mailman2sympa) in a Database
#
# Copyright (c) 2002 Andreas de Pretis for
#                    Uptime Systemloesungen GmbH and
#                    t0 Institut fuer neue Kulturtechnologien
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#
# Pragmas
#
use strict;
use diagnostics;

#
# Classes
#
use DBI qw(:sql_types);

#
# Variables
#
my $listname;
my $dbh;
my $dbh_csv;
my $date_func;
my $query_users_ins;
my $query_users_sel;
my $query_subscribers;
my $query_csv;
my $sth_csv;
my $sth_users_ins;
my $sth_users_sel;
my $sth_subscribers;
my $resetted = {};

#
# Load ENV
#
&getConfig();

#
# connect to database
#
&db_connect();

#
# Clean talbes
#
if (exists($ENV{'CLEANTABLES'}) && defined($ENV{'CLEANTABLES'}) && $ENV{'CLEANTABLES'} == 1) {
	print STDOUT "#### deleteing user_table and subscriber_table ####\n";

	eval {
		$dbh->do("DELETE FROM user_table");
		$dbh->do("DELETE FROM subscriber_table");
	};

	if ($@) {
		$dbh->rollback();
		die "Error: $@\n";
	} else {
		$dbh->commit();
	}
}

print STDOUT "#### importing ####\n";

#
# SQL-Statements
#
eval {
	#
	# CSV-Users
	#
	$query_csv	= qq!
		SELECT email_user, password_user from users
		!;
	$sth_csv	= $dbh_csv->prepare($query_csv);

	#
	# Users
	#
	$query_users_sel	= qq!
		SELECT *
		  FROM user_table
		 WHERE email_user like ?
		!;
	$sth_users_sel		= $dbh->prepare($query_users_sel);
		
	$query_users_ins	= qq!
		INSERT INTO user_table (email_user, password_user)
		     VALUES (?, ?)
		!;
	$sth_users_ins		= $dbh->prepare($query_users_ins);


	#
	# Subscribers
	#
	$query_subscribers	= qq!
		INSERT INTO subscriber_table (list_subscriber, user_subscriber, date_subscriber, reception_subscriber, visibility_subscriber)
		     VALUES (?, ?, ?, ?, ?)
		!;
	$sth_subscribers	= $dbh->prepare($query_subscribers);

	#
	# Execute
	#
	$sth_csv->execute();
	
	while (my ($email, $password) = $sth_csv->fetchrow_array()) {
		#
		# Check for existing user
		#
		if (!exists($resetted->{$email}) && !defined($resetted->{$email})) {
			$sth_users_sel->bind_param(1, $email, SQL_VARCHAR);
			$sth_users_sel->execute();
	
			if ($sth_users_sel->rows()) {
				my $sth;
				my $query;
			
				print STDOUT " - user '$email' already exists (setting password to null)\n";
			
				$query = qq!
					UPDATE user_table
					   SET password_user = null
					 WHERE email_user like ?
					!;
				
				$sth = $dbh->prepare($query);
				$sth->bind_param(1, $email, SQL_VARCHAR);
				$sth->execute();
				$sth->finish();
				$dbh->commit();
				
				$resetted->{$email} = 1;
			} else {
				$sth_users_ins->bind_param(1, $email, SQL_VARCHAR);
				$sth_users_ins->bind_param(2, $password, SQL_VARCHAR);
				$sth_users_ins->execute();
			}
		}
	}

	$sth_csv->finish();

	#
	# CSV-Subscribers
	#
	$query_csv	= qq!
		SELECT list_subscriber, user_subscriber, date_subscriber, reception_subscriber, visibility_subscriber from subscribers
		!;
	$sth_csv	= $dbh_csv->prepare($query_csv);

	#
	# Execute
	#
	$sth_csv->execute();

	while (my ($list, $user, $date, $visiblity, $reception) = $sth_csv->fetchrow_array()) {
		$sth_subscribers->bind_param(1, $list, SQL_VARCHAR);
		$sth_subscribers->bind_param(2, $user, SQL_VARCHAR);
		$sth_subscribers->bind_param(3, $date, SQL_VARCHAR);
		$sth_subscribers->bind_param(4, $reception, SQL_VARCHAR);
		$sth_subscribers->bind_param(5, $visiblity, SQL_VARCHAR);
		$sth_subscribers->execute();
	}

	$sth_csv->finish();
	$sth_users_ins->finish();
	$sth_users_sel->finish();
	$sth_subscribers->finish();
};

#
# Exception-Handling
#
if ($@) {
	$dbh->rollback();
	$sth_csv->finish();
	$sth_users_ins->finish();
	$sth_users_sel->finish();
	$sth_subscribers->finish();
	die "Error: $@";
} else {
	$dbh->commit();
}

print "\n\nPasswords resetted to null: " . keys (%{$resetted}) . "\n";

#
# disconnect from database
#
&db_disconnect();

#
# getConfig
#
sub getConfig
	{
	my @env 	= qx|. ./conf/mailman2sympa.conf;set|;

	#
	# Import ENV from mailman2sympa.conf into Perl-ENV
	#
        foreach my $env (@env)
        	{
                chomp $env;
                $env =~ /^\s*(.*?)\s*=\s*(.*)/;
                
                unless (exists($ENV{$1}) || $1 eq 'SHELLOPTS')
                	{
                        my $name	= $1;
                        my $value	= $2;
                        
                        if ($value =~ /^\'(.*)\'$/)
                        	{
                                $value = $1;
                        	}
                        	
                        $ENV{$name} = $value;
                	}
                }
	}

#
# db_connect
#
sub db_connect {
	my $db_type	= $ENV{'SGBD'};
	my $db_name	= $ENV{'DATABASE'};
	my $db_host	= $ENV{'HOSTDB'};
	my $db_user	= $ENV{'USERDB'};
	my $db_passwd	= $ENV{'PASSWORDDB'};

	unless ($dbh = DBI->connect("dbi:$db_type:dbname=$db_name;host=$db_host", $db_user, $db_passwd, { RaiseError => 1, AutoCommit => 0 })) {
		die "Can't connect to Database :", $DBI::errstr;
	}

	unless ($dbh_csv = DBI->connect("dbi:CSV:f_dir=$ENV{'WDIR'}/csv", { RaiseError => 1, AutoCommit => 0 })) {
		die "Can't connect to CSV-Directory :", $DBI::errstr;
	}

	$dbh_csv->{'csv_tables'}->{'users'}= {
		'file'			=> 'import_users.csv',
		'sep_char' 		=> ";",
		'eol'			=> "\n",
		'col_names'		=> [
			'email_user',
			'password_user',
			],
		};

	$dbh_csv->{'csv_tables'}->{'subscribers'} = {
		'file'			=> 'import_subscribers.csv',
		'sep_char' 		=> ";",
		'eol'			=> "\n",
		'col_names'		=> [
			'list_subscriber',
			'user_subscriber',
			'date_subscriber',
			'visibility_subscriber',
			'reception_subscriber',
			'bounce_subscriber',
			'comment_subscriber',
			'subscribed_subscriber',
			'included_subscriber',
			'include_sources_subscriber',
			],
		};
}

#
# db_disconnect
#
sub db_disconnect {
	$dbh->disconnect();
	$dbh_csv->disconnect();
}
